﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using IntercanviCases.Models;
using IntercanviCases.DAL;

namespace IntercanviCases.Controllers
{
    public class CercaController : Controller
    {
        private IntercanviCasesContext db = new IntercanviCasesContext();
        private List<Poblacio> poblacionsLst = new List<Poblacio>();
        private List<Regio> regionsLst = new List<Regio>();
        private List<Pais> paisosLst = new List<Pais>();

        //
        // GET: /Cerca/

        public ActionResult Index(bool? animalsPermesos, bool? cotxeDisponible, DateTime? dataInici, DateTime? dataFi, int? PaisViuID, int? RegioViuID, int? poblacioViuID)
        {
            var ofertes = from o in db.ofertes
                          select o;
            ofertes.Include(o => o.poblacio).Include(o => o.perfil);

            llistatPoblacions();

            ViewBag.pais = GetCountrySelectList(paisosLst); 

            //Procés de valors d'entrada

            if (PaisViuID.HasValue)
            {
                ofertes = ofertes.Where(o => o.poblacio.regio.paisID == PaisViuID);
            }
            if (RegioViuID.HasValue)
            {
                ofertes = ofertes.Where(o => o.poblacio.regioID == RegioViuID);
            }
            if (poblacioViuID.HasValue)
            {
                ofertes = ofertes.Where(o => o.poblacioID == poblacioViuID);
            }

            if (animalsPermesos==true)
            {
                ofertes = ofertes.Where(o => o.animalsPermesos == true);
            }
            else if (animalsPermesos == false || animalsPermesos == null)
            { animalsPermesos = false; }

            if (cotxeDisponible==true)
            {
                ofertes = ofertes.Where(o => o.cotxeDisponible == true);
            }
            else if (cotxeDisponible == false || cotxeDisponible == null)
            { cotxeDisponible = false; }
            
            if (!String.IsNullOrEmpty(Convert.ToString(dataFi)))
            {
                ofertes = ofertes.Where(o => o.dataFi <= dataFi);
            }

            if (!String.IsNullOrEmpty(Convert.ToString(dataInici)))
            {
                ofertes = ofertes.Where(o => o.dataInici >= dataInici );
            }
            

            ViewBag.animalsPermesos = animalsPermesos;
            ViewBag.cotxeDisponible = cotxeDisponible;

            return View(ofertes.ToList());
        }

        //
        // GET: /Cerca/Details/5

        public ActionResult Details(int id = 0)
        {
            Oferta oferta = db.ofertes.Find(id);
            if (oferta == null)
            {
                return HttpNotFound();
            }
            return View(oferta);
        }

         protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }


        //Cal cridar la funcio cada cop que volguem refrescar els combos de poblacions pq en fer servir AJAX es torna a carregar el controlador i s'esborren les variables globals.
        private void llistatPoblacions()
        {
            var ofertes = from o in db.ofertes
                          select o;
            ofertes.Include(o => o.poblacio).Include(o => o.perfil);

            foreach (Oferta o in ofertes)
            {
                poblacionsLst.Add(o.poblacio);
            }

            foreach (Poblacio p in poblacionsLst)
            {
                regionsLst.Add(p.regio);
            }


            foreach (Regio r in regionsLst)
            {
                paisosLst.Add(r.pais);
            }
        }

        public SelectList GetCountrySelectList()
        {
            var paisos = db.paisos;
            return new SelectList(paisos.ToArray(),
                        "paisID",
                        "nom");
        }

        public SelectList GetCountrySelectList(List<Pais> paisos)
        {
            return new SelectList(paisos.Distinct(),
                        "paisID",
                        "nom");
        }

        public ActionResult RegioList(int ID)
        {
            int paisID = ID;
            //s'omplen les regions limitades a les ofertes del perfil actual, no totes
            llistatPoblacions();
            
            var region = from r in regionsLst
                         where r.paisID == paisID
                         select r;

            if (HttpContext.Request.IsAjaxRequest())
                return Json(new SelectList(
                                region.Distinct(),
                                "regioID",
                                "nom")
                           , JsonRequestBehavior.AllowGet);

            return RedirectToAction("Index");
        }

        public ActionResult PoblacioList(int ID)
        {
            int regioID = ID;
            llistatPoblacions();
            //var poblacions = db.poblacions;
            var poblacio = from p in poblacionsLst
                           where p.regioID == regioID
                           select p;

            if (HttpContext.Request.IsAjaxRequest())
                return Json(new SelectList(
                                poblacio.Distinct(),
                                "poblacioID",
                                "nom")
                           , JsonRequestBehavior.AllowGet);

            return RedirectToAction("Index");
        }  
    }
}