﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using IntercanviCases.Models;
using IntercanviCases.DAL;
using IntercanviCases.FuncionsComunes;

namespace IntercanviCases.Controllers
{
    [Authorize]
    public class IntercanviController : Controller
    {
        private IntercanviCasesContext db = new IntercanviCasesContext();

        //
        // GET: /Intercanvi/

        public ActionResult Index()
        {
            var intercanvis = db.Intercanvis.Include(i => i.oferta1).Include(i => i.oferta2);
            return View(intercanvis.ToList());
        }

        //
        // GET: /Intercanvi/Details/5

        public ActionResult Details(int id = 0)
        {
            Intercanvi intercanvi = db.Intercanvis.Find(id);
            if (intercanvi == null)
            {
                return HttpNotFound();
            }
            return View(intercanvi);
        }

        //
        // GET: /Intercanvi/Create

        public ActionResult Create(int id = 0)
        {            
            Oferta oferta2 = db.ofertes.Find(id);
            var oferta1 = db.ofertes.Where(o => o.perfil.userName == User.Identity.Name);
            ViewBag.oferta1ID = new SelectList(oferta1, "ofertaID", "descCurta");
            ViewBag.oferta2 = oferta2; 
            Intercanvi intercanvi = new Intercanvi();
            intercanvi.dataInici2 = oferta2.dataInici;
            intercanvi.dataFi2 = oferta2.dataFi;
            intercanvi.oferta2ID = oferta2.ofertaID;

            //TODO: calcul de dies coincidents
            TimeSpan remDays = intercanvi.dataInici1.Subtract(intercanvi.dataInici2);
            int days = remDays.Days;

            return View(intercanvi);
        }



        //
        // POST: /Intercanvi/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Intercanvi intercanvi)
        {
            Oferta oferta2 = db.ofertes.Find(intercanvi.oferta2ID);
            Oferta oferta1 = db.ofertes.Find(intercanvi.oferta1ID);

            string missatge = "<p>Ha començat l'intercanvi d'un usuari</p>"
                + "<p>Visita la seva oferta! "
                + oferta1.descCurta + " "
                + "http://" + Request.Url.Authority.ToString() + "/Cerca/Details/" + intercanvi.oferta1ID.ToString();

            funcions.SendEmail(oferta2.perfil.email, "Intercanvi iniciat", missatge);
            
            if (ModelState.IsValid)
            {
                db.Intercanvis.Add(intercanvi);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            ViewBag.oferta1ID = new SelectList(db.ofertes, "ofertaID", "descCurta", intercanvi.oferta1ID);
            ViewBag.oferta2ID = new SelectList(db.ofertes, "ofertaID", "descCurta", intercanvi.oferta2ID);
            return View(intercanvi);
        }

        //
        // GET: /Intercanvi/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Intercanvi intercanvi = db.Intercanvis.Find(id);
            ViewBag.userName1 = intercanvi.oferta1.perfil.userName;
            ViewBag.userName2 = intercanvi.oferta2.perfil.userName;
            ViewBag.userLogin = User.Identity.Name;
            ViewBag.oferta1descCurta = intercanvi.oferta1.descCurta;
            ViewBag.oferta2descCurta = intercanvi.oferta2.descCurta;

            //Oferta oferta2 = db.ofertes.Find(id);
            var oferta1 = db.ofertes.Where(o => o.perfil.userName == intercanvi.oferta1.perfil.userName);
            var oferta2 = db.ofertes.Where(o => o.perfil.userName == intercanvi.oferta2.perfil.userName);
            ViewBag.oferta1ID = new SelectList(oferta1, "ofertaID", "descCurta", intercanvi.oferta1ID);
            ViewBag.oferta2ID = new SelectList(oferta2, "ofertaID", "descCurta", intercanvi.oferta2ID);
            ViewBag.oferta2 = oferta2; 

            if (intercanvi == null)
            {
                return HttpNotFound();
            }
           // ViewBag.oferta1ID = new SelectList(db.ofertes, "ofertaID", "descCurta", intercanvi.oferta1ID);
            //ViewBag.oferta2ID = new SelectList(db.ofertes, "ofertaID", "descCurta", intercanvi.oferta2ID);
            return View(intercanvi);
        }

        //
        // POST: /Intercanvi/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Intercanvi intercanvi)
        {
            if (ModelState.IsValid)
            {
                db.Entry(intercanvi).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.oferta1ID = new SelectList(db.ofertes, "ofertaID", "descCurta", intercanvi.oferta1ID);
            ViewBag.oferta2ID = new SelectList(db.ofertes, "ofertaID", "descCurta", intercanvi.oferta2ID);
            return View(intercanvi);
        }

        //
        // GET: /Intercanvi/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Intercanvi intercanvi = db.Intercanvis.Find(id);
            if (intercanvi == null)
            {
                return HttpNotFound();
            }
            return View(intercanvi);
        }

        //
        // POST: /Intercanvi/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Intercanvi intercanvi = db.Intercanvis.Find(id);
            db.Intercanvis.Remove(intercanvi);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        public ActionResult ofertaSel(int ID)
        {
            int ofertaID = ID;

            var ofertes = db.ofertes;
            var oferta = from o in ofertes.ToList()
                         where o.ofertaID == ofertaID
                         select o;

            var cad = oferta.First().dataInici.ToShortDateString() + "," + oferta.First().dataFi.ToShortDateString();

            if (HttpContext.Request.IsAjaxRequest())
                return Content(cad, "text/html");

            return RedirectToAction("Index");
        }
    }
}