﻿using System;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web.Mvc;
using IntercanviCases.Models;
using IntercanviCases.DAL;

namespace IntercanviCases.Controllers
{
    [Authorize]
    public class MissatgeController : Controller
    {
        private IntercanviCasesContext db = new IntercanviCasesContext();

        //
        // GET: /Missatge/

        public ActionResult Index()
        {
            //var missatges = db.missatges.Include(m => m.oferta);
            //return View(missatges.ToList());
            return RedirectToAction("Rebuts");
        }

        public ActionResult Rebuts()
        {
            var missatges = db.missatges.Include(m => m.oferta);

            return View(missatges.ToList().Where(m => m.receptor.userName == User.Identity.Name));
        }

        public ActionResult Enviats()
        {
            var missatges = db.missatges.Include(m => m.oferta);

            return View(missatges.ToList().Where(m => m.emissor.userName == User.Identity.Name));
        }

        //
        // GET: /Missatge/Details/5

        public ActionResult Details(int id = 0)
        {
            Missatge missatge = db.missatges.Find(id);

            //si la oferta referent del missatge no es de l'usuari que l'esta veient no se li mostra l'accio d'iniciar intercanvi
            //TODO: que no surti el link si ja està un intercanvi iniciat
            ViewBag.mostrarAccioIntercanvi = !(missatge.oferta.perfil.userName == User.Identity.Name);
            if (missatge == null)
            {
                return HttpNotFound();
            }
            return View(missatge);
        }

        //
        // GET: /Missatge/Create

        [Authorize]
        public ActionResult Create() 
        {
            //Això podria ser per l'intercanvi, per passar ja en el missatge la oferta del emissor.
            //  ViewBag.ofertaIDemissor = new SelectList(db.ofertes.Where(o => o.perfil.userName == User.Identity.Name), "ofertaIDemissor", "descCurta");
            //new SelectList(db.ofertes, "ofertaID", "descCurta", missatge.ofertaID);
            //ViewBag.poblacioViuID = new SelectList(db.poblacions, "poblacioID", "nom", perfil.poblacioViuID);
            return View();
        }

        //
        // POST: /Missatge/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Missatge missatge)
        {
            missatge.receptor = db.ofertes.Find(missatge.ofertaID).perfil;
            missatge.emissor = db.perfils.Where(p => p.userName == User.Identity.Name).First();
            missatge.data = DateTime.Now;

            if (ModelState.IsValid)
            {
                db.missatges.Add(missatge);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
                       
            return View(missatge);
        }

        public ActionResult Respondre(int id = 0)
        {
            Missatge missatge = db.missatges.Find(id);
            if (missatge == null)
            {
                return HttpNotFound();
            }
            //la resposta es una copia de l'original pero sense contingut, nomes es mostra a titol informatiu
            ViewBag.copiaContingut = missatge.contingut;
            missatge.contingut = "";
            return View(missatge);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Respondre(Missatge missatge)
        {
            //En respondre un missatge el missatge que aqui rebem és un missatge nou per tant cal recuperar i afegir  alguns valors.
            missatge.receptor = db.missatges.Find(missatge.missatgeID).emissor; 
            missatge.emissor = db.perfils.Where(p => p.userName == User.Identity.Name).First();
            missatge.ofertaID = db.missatges.Find(missatge.missatgeID).ofertaID;
            missatge.data = DateTime.Now;

            if (ModelState.IsValid)
            {
                db.missatges.Add(missatge);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            missatge.contingut = ""; 
            return View(missatge);
        }

        //
        // GET: /Missatge/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Missatge missatge = db.missatges.Find(id);
            if (missatge == null)
            {
                return HttpNotFound();
            }
            ViewBag.ofertaID = new SelectList(db.ofertes, "ofertaID", "descCurta", missatge.ofertaID);
            return View(missatge);
        }

        //
        // POST: /Missatge/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Missatge missatge)
        {
            if (ModelState.IsValid)
            {
                db.Entry(missatge).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.ofertaID = new SelectList(db.ofertes, "ofertaID", "descCurta", missatge.ofertaID);
            return View(missatge);
        }

        //
        // GET: /Missatge/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Missatge missatge = db.missatges.Find(id);
            if (missatge == null)
            {
                return HttpNotFound();
            }
            return View(missatge);
        }

        //
        // POST: /Missatge/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Missatge missatge = db.missatges.Find(id);
            db.missatges.Remove(missatge);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}