﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using IntercanviCases.Models;
using IntercanviCases.DAL;
using IntercanviCases.FuncionsComunes;

namespace IntercanviCases.Controllers
{
    [Authorize]
    public class OfertaController : Controller
    {
        private IntercanviCasesContext db = new IntercanviCasesContext();
        private List<Poblacio> poblacionsLst = new List<Poblacio>();
        private List<Regio> regionsLst = new List<Regio>();
        private List<Pais> paisosLst = new List<Pais>();

        //
        // GET: /Oferta/

        public ActionResult Index()
        {
            var ofertes = db.ofertes.Include(o => o.poblacio).Include(o => o.perfil);
            var ofertesPerfil = from o in ofertes.ToList()
                           where o.perfil.userName == User.Identity.Name
                           select o;
            return View(ofertesPerfil.ToList());
        }

        //
        // GET: /Oferta/Details/5

        public ActionResult Details(int id = 0)
        {
            Oferta oferta = db.ofertes.Find(id);
            if (oferta == null || oferta.perfil.userName != User.Identity.Name)
            {
                return HttpNotFound();
            }
            return View(oferta);
        }

        //
        // GET: /Oferta/Create

        public ActionResult Create(int id = 0)
        {       
            ViewBag.Pais = GetCountrySelectList();
            ViewBag.perfilID = id; 
            return View();
        }

        //
        // POST: /Oferta/Create
        // a més, s'envia un email als perfils interessats

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Oferta oferta)
        {
            if (ModelState.IsValid)
            {
                //Recollim el perfil id que ens passa el perfil
                oferta.perfilID = (int)Session["perfilID"]; 
                oferta.dataActualitzat = DateTime.Now;
                db.ofertes.Add(oferta);
                db.SaveChanges();

                List<Perfil> perfils = db.perfils.Where(p => p.poblacioPrefereixID == oferta.poblacioID).ToList();

               foreach (Perfil p in perfils)
                {
                    string missatge = "<p>"+ p.nom +", s'ha creat una ofeta a una població que et mola!</p>"
                   + "<p>Visita la seva oferta! "
                   + oferta.descCurta + " "
                   + "http://" + Request.Url.Authority.ToString() + "/Cerca/Details/" + oferta.ofertaID.ToString();

                    funcions.SendEmail(p.email, "Una oferta interessant", missatge);
                }

                return RedirectToAction("Index");
            }

            ViewBag.poblacioID = new SelectList(db.poblacions, "poblacioID", "nom", oferta.poblacioID);
            ViewBag.perfilID = new SelectList(db.perfils, "perfilID", "nom", oferta.perfilID);
            return View(oferta);
        }

        //
        // GET: /Oferta/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Oferta oferta = db.ofertes.Find(id);
            if (oferta == null)
            {
                return HttpNotFound();
            }
           
            //TODO: EL select de poblacions no surt la poblacio per defecte
            ViewBag.perfilID = new SelectList(db.perfils, "perfilID", "nom", oferta.perfilID);
            ViewBag.paisID = new SelectList(db.paisos, "paisID", "nom", oferta.poblacio.regio.paisID);
            ViewBag.regioID = new SelectList(db.regions.Where(r => r.paisID == oferta.poblacio.regio.paisID), "regioID", "nom", oferta.poblacio.regioID);
            ViewBag.poblacioID = new SelectList(db.poblacions.Where(p => p.regioID == oferta.poblacio.regioID), "poblacioID", "nom", oferta.poblacioID);

            return View(oferta);
        }

        //
        // POST: /Oferta/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Oferta oferta)
        {
            oferta.dataActualitzat = DateTime.Now;
            if (ModelState.IsValid)
            {
                db.Entry(oferta).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.poblacioID = new SelectList(db.poblacions, "poblacioID", "nom", oferta.poblacioID);
            ViewBag.perfilID = new SelectList(db.perfils, "perfilID", "nom", oferta.perfilID);
            return View(oferta);
        }

        //
        // GET: /Oferta/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Oferta oferta = db.ofertes.Find(id);
            if (oferta == null)
            {
                return HttpNotFound();
            }
            return View(oferta);
        }

        //
        // POST: /Oferta/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Oferta oferta = db.ofertes.Find(id);
            db.ofertes.Remove(oferta);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        //Cal cridar la funcio cada cop que volguem refrescar els combos de poblacions pq en fer servir AJAX es torna a carregar el controlador i s'esborren les variables globals.
        private void llistatPoblacions()
        {
            var ofertes = from o in db.ofertes
                          select o;
            ofertes.Include(o => o.poblacio).Include(o => o.perfil);

            foreach (Oferta o in ofertes)
            {
                poblacionsLst.Add(o.poblacio);
            }

            foreach (Poblacio p in poblacionsLst)
            {
                regionsLst.Add(p.regio);
            }


            foreach (Regio r in regionsLst)
            {
                paisosLst.Add(r.pais);
            }
        }

        public SelectList GetCountrySelectList()
        {
            var paisos = db.paisos;
            return new SelectList(paisos.ToArray(),
                        "paisID",
                        "nom");
        }

        public SelectList GetCountrySelectList(List<Pais> paisos)
        {
            return new SelectList(paisos.Distinct(),
                        "paisID",
                        "nom");
        }

        public ActionResult RegioList(int ID)
        {
            int paisID = ID;

            llistatPoblacions();

            //A diferencia del controlador de cerca, aqui les regions poden estar limitades o no depenent de si estem editant o creant
            if (regionsLst.Any())
                regionsLst = db.regions.ToList();

            var region = from r in regionsLst
                         where r.paisID == paisID
                         select r;

            if (HttpContext.Request.IsAjaxRequest())
                return Json(new SelectList(
                                region.Distinct(),
                                "regioID",
                                "nom")
                           , JsonRequestBehavior.AllowGet);

            return RedirectToAction("Index");
        }

        public ActionResult PoblacioList(int ID)
        {
            int regioID = ID;

            llistatPoblacions();

            if (poblacionsLst.Any())
                poblacionsLst = db.poblacions.ToList();

            var poblacio = from p in poblacionsLst
                           where p.regioID == regioID
                           select p;

            if (HttpContext.Request.IsAjaxRequest())
                return Json(new SelectList(
                                poblacio.Distinct(),
                                "poblacioID",
                                "nom")
                           , JsonRequestBehavior.AllowGet);

            return RedirectToAction("Index");
        }

    }
}