﻿using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using IntercanviCases.Models;
using IntercanviCases.DAL;

namespace IntercanviCases.Controllers
{

    [Authorize]
    public class PerfilController : Controller
    {
        private IntercanviCasesContext db = new IntercanviCasesContext();
        private List<Poblacio> poblacionsLst = new List<Poblacio>();
        private List<Regio> regionsLst = new List<Regio>();
        private List<Pais> paisosLst = new List<Pais>();
        //
        // GET: /Perfil/

        
        public ActionResult Index()
        {
            var perfils = db.perfils.Include(p => p.poblacioViu).Include(p => p.poblacioPrefereix);
            var perfilUsuari = from p in perfils.ToList()
                                where p.userName == User.Identity.Name
                                select p;
            Session["perfilID"] = -1;

            if (perfilUsuari.Any())
            {
                Session["perfilID"] = perfilUsuari.First().perfilID;
            }
            return View(perfilUsuari.ToList());
        }

        //
        // GET: /Perfil/Details/5
        //Nomes pot veure el seu perfil l'usuari registrat coincident

        public ActionResult Details(int id = 0)
        {
            Perfil perfil = db.perfils.Find(id);
            Session["perfilID"] = id;
            if (perfil == null || perfil.userName != User.Identity.Name)
            {
                return HttpNotFound();
            }
            return View(perfil);
        }

        //
        // GET: /Perfil/Create

        public ActionResult Create()
        {
            ViewBag.Pais = GetCountrySelectList();
            return View();
        }

        //
        // POST: /Perfil/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Perfil perfil)
        {
            //S'afegeix el nom d'usuari registrat al perfil de l'usuari
            perfil.userName = User.Identity.Name;

            if (ModelState.IsValid)
            {
                db.perfils.Add(perfil);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

           // ViewBag.poblacioViuID = new SelectList(db.poblacions, "poblacioID", "nom", perfil.poblacioViuID);
           // ViewBag.poblacioPrefereixID = new SelectList(db.poblacions, "poblacioID", "nom", perfil.poblacioPrefereixID);
            ViewBag.Pais = GetCountrySelectList();
            return View(perfil);
        }

        //
        // GET: /Perfil/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Perfil perfil = db.perfils.Find(id);
            if (perfil == null)
            {
                return HttpNotFound();
            }
            llistatPoblacions();

            ViewBag.PaisViu = new SelectList(db.paisos.OrderBy(p => p.nom), "paisID", "nom", perfil.poblacioViu.regio.paisID);
            ViewBag.RegionsViu = new SelectList(db.regions.Where(r => r.paisID == perfil.poblacioViu.regio.paisID).OrderBy(r => r.nom), "RegioID", "nom", perfil.poblacioViu.regioID);
            ViewBag.PoblacionsViu = new SelectList(db.poblacions.Where(p => p.regioID == perfil.poblacioViu.regioID).OrderBy(p => p.nom), "PoblacioID", "nom", perfil.poblacioViuID);

            ViewBag.PaisPref = new SelectList(db.paisos.OrderBy(p => p.nom), "paisID", "nom", perfil.poblacioPrefereix.regio.paisID);
            ViewBag.RegionsPref = new SelectList(db.regions.Where(r => r.paisID == perfil.poblacioPrefereix.regio.paisID).OrderBy(r => r.nom), "RegioID", "nom", perfil.poblacioPrefereix.regioID);
            ViewBag.PoblacionsPref = new SelectList(db.poblacions.Where(p => p.regioID == perfil.poblacioPrefereix.regioID).OrderBy(p => p.nom), "PoblacioID", "nom", perfil.poblacioPrefereixID);
            return View(perfil);
        }

        //
        // POST: /Perfil/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Perfil perfil)
        {
            perfil.userName = User.Identity.Name;

            if (ModelState.IsValid)
            {
                db.Entry(perfil).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.poblacioViuID = new SelectList(db.poblacions, "poblacioID", "nom", perfil.poblacioViuID);
            ViewBag.poblacioPrefereixID = new SelectList(db.poblacions, "poblacioID", "nom", perfil.poblacioPrefereixID);
            return View(perfil);
        }

        //
        // GET: /Perfil/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Perfil perfil = db.perfils.Find(id);
            if (perfil == null)
            {
                return HttpNotFound();
            }
            return View(perfil);
        }

        //
        // POST: /Perfil/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Perfil perfil = db.perfils.Find(id);
            db.perfils.Remove(perfil);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        //Cal cridar la funcio cada cop que volguem refrescar els combos de poblacions pq en fer servir AJAX es torna a carregar el controlador i s'esborren les variables globals.
        private void llistatPoblacions()
        {
            var ofertes = from o in db.ofertes
                          select o;
            ofertes.Include(o => o.poblacio).Include(o => o.perfil);

            foreach (Oferta o in ofertes)
            {
                poblacionsLst.Add(o.poblacio);
            }

            foreach (Poblacio p in poblacionsLst)
            {
                regionsLst.Add(p.regio);
            }


            foreach (Regio r in regionsLst)
            {
                paisosLst.Add(r.pais);
            }
        }

        public SelectList GetCountrySelectList()
        {
            var paisos = db.paisos;
            return new SelectList(paisos.ToArray(),
                        "paisID",
                        "nom");
        }

        public SelectList GetCountrySelectList(List<Pais> paisos)
        {
            return new SelectList(paisos.Distinct(),
                        "paisID",
                        "nom");
        }

        public ActionResult RegioList(int ID)
        {
            int paisID = ID;

            llistatPoblacions();

            //A diferencia del controlador de cerca, aqui les regions poden estar limitades o no depenent de si estem editant o creant
            if (regionsLst.Any())
                regionsLst = db.regions.ToList();

            var region = from r in regionsLst
                         where r.paisID == paisID
                         orderby r.nom
                         select r;

            if (HttpContext.Request.IsAjaxRequest())
                return Json(new SelectList(
                                region.Distinct(),
                                "regioID",
                                "nom")
                           , JsonRequestBehavior.AllowGet);

            return RedirectToAction("Index");
        }

        public ActionResult PoblacioList(int ID)
        {
            int regioID = ID;

            llistatPoblacions();

            if (poblacionsLst.Any())
                poblacionsLst = db.poblacions.ToList();

            var poblacio = from p in poblacionsLst
                           where p.regioID == regioID
                           orderby p.nom
                           select p;

            if (HttpContext.Request.IsAjaxRequest())
                return Json(new SelectList(
                                poblacio.Distinct(),
                                "poblacioID",
                                "nom")
                           , JsonRequestBehavior.AllowGet);

            return RedirectToAction("Index");
        } 
    }
}