﻿using IntercanviCases.Models;
using System.Data.Entity;
using System.Data.Entity.ModelConfiguration.Conventions;

//Data Access Layer
namespace IntercanviCases.DAL
{
    public class IntercanviCasesContext : DbContext
    {
        public DbSet<Oferta> ofertes { get; set; }
        public DbSet<Pais> paisos { get; set; }
        public DbSet<Poblacio> poblacions { get; set; }
        public DbSet<Regio> regions { get; set; }
        public DbSet<Perfil> perfils { get; set; }
        public DbSet<Missatge> missatges { get; set; }
        public DbSet<Extra> extres { get; set; }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
           //Trec la convenció de que em faci plurals perque queda fatal: regios, poblacios, etc.
            modelBuilder.Conventions.Remove<PluralizingTableNameConvention>();

            //cal indicar la relació entre els dos models Oferta i Extra, perquè es crei la taula intermitja
            modelBuilder.Entity<Oferta>()
                .HasMany(o => o.extres).WithMany(e => e.ofertes)
                .Map(t => t.MapLeftKey("ofertaID").MapRightKey("extraID").ToTable("ExtraOferta"));

            modelBuilder.Entity<Perfil>().HasRequired(d => d.poblacioViu).WithMany().WillCascadeOnDelete(false);
        }

        public DbSet<Intercanvi> Intercanvis { get; set; }
    }
}