﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data.Entity;
using System.Globalization;
using System.Web.Security;

namespace IntercanviCases.Models
{
    public class UsersContext : DbContext
    {
        public UsersContext()
            : base("DefaultConnection")
        {
        }

        public DbSet<UserProfile> UserProfiles { get; set; }
    }

    [Table("UserProfile")]
    public class UserProfile
    {
        [Key]
        [DatabaseGeneratedAttribute(DatabaseGeneratedOption.Identity)]
        public int UserId { get; set; }
        public string UserName { get; set; }
    }

    public class RegisterExternalLoginModel
    {
        [Required]
        [Display(Name = "Nom d'usuari")]
        public string UserName { get; set; }

        public string ExternalLoginData { get; set; }
    }

    public class LocalPasswordModel
    {
        [Required]
        [DataType(DataType.Password)]
        [Display(Name = "Contrasenya actual")]
        public string OldPassword { get; set; }

        [Required]
        [StringLength(100, ErrorMessage = "El nombre de caracters de {0} ha de ser com a mínim de {2}.", MinimumLength = 6)]
        [DataType(DataType.Password)]
        [Display(Name = "Nova contrasenya")]
        public string NewPassword { get; set; }

        [DataType(DataType.Password)]
        [Display(Name = "Confirmar la nova contrasenya")]
        [Compare("NewPassword", ErrorMessage = "La nova contrasenya i la contrasenya de confirmació no coincideixen .")]
        public string ConfirmPassword { get; set; }
    }

    public class LoginModel
    {
        [Required]
        [Display(Name = "Nom d'usuari")]
        public string UserName { get; set; }

        [Required]
        [DataType(DataType.Password)]
        [Display(Name = "Contrasenya")]
        public string Password { get; set; }

        [Display(Name = "Recordar compte?")]
        public bool RememberMe { get; set; }
    }

    public class RegisterModel
    {
        [Required]
        [Display(Name = "Nom d'usuari")]
        public string UserName { get; set; }

        [Required]
        [StringLength(100, ErrorMessage = "El nombre de caracters de {0} ha de ser com a mínim de {2}.", MinimumLength = 6)]
        [DataType(DataType.Password)]
        [Display(Name = "Contrasenya")]
        public string Password { get; set; }

        [DataType(DataType.Password)]
        [Display(Name = "Confirmar contrasenya")]
        [Compare("Password", ErrorMessage = "La nova contrasenya i la contrasenya de confirmació no coincideixen .")]
        public string ConfirmPassword { get; set; }
    }

    public class ExternalLogin
    {
        public string Provider { get; set; }
        public string ProviderDisplayName { get; set; }
        public string ProviderUserId { get; set; }
    }
}
