﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace IntercanviCases.Models
{
    public class Perfil
    {
        public int perfilID { get; set; }

        public string userName { get; set; }
        
        [Required]
        [Display(Name="Nom")]
        public string nom { get; set; }

        [Required]
        [Display(Name = "Cognoms")]
        public string cognoms { get; set; }

        [Display(Name = "Email")]
        [Required(ErrorMessage = "Cal introduir l'adreça d'email")]
        [EmailAddress(ErrorMessage = "Adreça d'email incorrecta")]
        public string email { get; set; }

        [Display(Name = "Descripció")]
        [DataType(DataType.MultilineText)]
        public string descripcio { get; set; }

        //Aquest camp permetria a un administrador mostrar o no les ofertes
        [ScaffoldColumn(false)]
        public bool ofertesActivades { get; set; }

        [Display(Name="Som fumadors")]
        public bool fumadors { get; set; }

        [Display (Name="Tenim animals")]
        public bool animals { get; set; }

        //Els dos atributs de població del perfil, viu i prefereix fan referència a la clau primària de població, això s'indica aqui i al model de Població amb la InverseProperty
        // podem definir la clau forània, si no, la migració posa una cosa com poblacioViu_poblacioID, igualment o ha de ser nullable o cal afegir una regla al context WillCascadeOnDelete(false)
        public int poblacioViuID { get; set; }
        [ForeignKey("poblacioViuID")] 
        public virtual Poblacio poblacioViu { get; set; }


        public int? poblacioPrefereixID { get; set; }
        [ForeignKey("poblacioPrefereixID")]
        public virtual Poblacio poblacioPrefereix { get; set; }

        //un perfil té una llista d'ofertes
        public virtual ICollection<Oferta> ofertes { get; set; }

        //Similar als atributs de població del perfil
        [InverseProperty("emissor")]
        public virtual ICollection<Missatge> missatgesEnviats { get; set; }
        [InverseProperty("receptor")]
        public virtual ICollection<Missatge> missatgesRebuts { get; set; }


    }
}